"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ContainerController = exports.runnerControllerOptionsDefault = void 0;
const child_process_1 = require("child_process");
const crypto_1 = __importDefault(require("crypto"));
const axios_1 = __importDefault(require("axios"));
const get_port_1 = __importDefault(require("get-port"));
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:container-controller');
exports.runnerControllerOptionsDefault = {
    gqlURN: 'deep_deeplinks_1:3006',
    network: 'deep_network',
    handlersHash: {},
};
class ContainerController {
    constructor(options) {
        this.handlersHash = {};
        this.network = (options === null || options === void 0 ? void 0 : options.network) || exports.runnerControllerOptionsDefault.network;
        this.gqlURN = (options === null || options === void 0 ? void 0 : options.gqlURN) || exports.runnerControllerOptionsDefault.gqlURN;
        this.handlersHash = (options === null || options === void 0 ? void 0 : options.handlersHash) || exports.runnerControllerOptionsDefault.handlersHash;
    }
    ;
    newContainer(options) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const { handler, forcePort, forceName, forceRestart, publish } = options;
            const { network, handlersHash, gqlURN } = this;
            debug('options, network, handlersHash', { options, network, handlersHash });
            const containerName = forceName || crypto_1.default.createHash('md5').update(handler).digest("hex");
            debug('containerName, forceName', { containerName, forceName });
            let container = yield this.findContainer(containerName);
            if (container)
                return container;
            let dockerPort = forcePort || (yield (0, get_port_1.default)());
            let dockerRunResult;
            let done = false;
            let count = 30;
            while (!done) {
                debug('count', { count });
                if (count < 0)
                    return { error: 'timeout findPort' };
                count--;
                try {
                    const command = `docker run -e PORT=${dockerPort} -e GQL_URN=${gqlURN} -e GQL_SSL=0 --name ${containerName} ${publish ? `-p ${dockerPort}:${dockerPort}` : `--expose ${dockerPort}`} --net ${network} -d ${handler}`;
                    debug('command', { command });
                    dockerRunResult = (0, child_process_1.execSync)(command).toString();
                    debug('dockerRunResult', { dockerRunResult });
                }
                catch (e) {
                    debug('error', e);
                    dockerRunResult = e.stderr;
                }
                if (dockerRunResult.indexOf('port is already allocated') !== -1) {
                    if (forcePort)
                        return { error: 'port is already allocated' };
                    continue;
                }
                else if (dockerRunResult.indexOf('is already in use by container') !== -1) {
                    if (!forceRestart)
                        return { error: 'is already in use by container' };
                    yield this._dropContainer(containerName);
                }
                else {
                    done = true;
                    const inspectResult = (0, child_process_1.execSync)(`docker inspect ${containerName}`).toString();
                    const inspectJSON = JSON.parse(inspectResult);
                    const ip = (_d = (_c = (_b = (_a = inspectJSON === null || inspectJSON === void 0 ? void 0 : inspectJSON[0]) === null || _a === void 0 ? void 0 : _a.NetworkSettings) === null || _b === void 0 ? void 0 : _b.Networks) === null || _c === void 0 ? void 0 : _c.deep_network) === null || _d === void 0 ? void 0 : _d.IPAddress;
                    container = { name: containerName, host: ip, port: dockerPort };
                    (0, child_process_1.execSync)(`npx wait-on http://${ip}:${dockerPort}/healthz`);
                    handlersHash[containerName] = container;
                }
                if (!done && !forcePort)
                    dockerPort = yield (0, get_port_1.default)();
            }
            return container;
        });
    }
    findContainer(containerName) {
        return __awaiter(this, void 0, void 0, function* () {
            const { handlersHash } = this;
            return handlersHash[containerName];
        });
    }
    _dropContainer(containerName) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield (0, child_process_1.execSync)(`docker stop ${containerName} && docker rm ${containerName}`).toString();
        });
    }
    dropContainer(container) {
        return __awaiter(this, void 0, void 0, function* () {
            const { handlersHash } = this;
            if (!handlersHash[container.name])
                return;
            let dockerStopResult = yield this._dropContainer(container.name);
            handlersHash[container.name] = undefined;
            debug('dockerStopResult', { dockerStopResult });
        });
    }
    initHandler(container) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const { host, port } = container;
            let initResult;
            try {
                const initRunner = `http://${host}:${port}/init`;
                debug('initRunner', { initRunner });
                initResult = yield axios_1.default.post(initRunner);
                debug('initResult', { initResult: initResult.toString() });
            }
            catch (e) {
                debug('error', e);
                return ({ error: e });
            }
            if ((_a = initResult === null || initResult === void 0 ? void 0 : initResult.data) === null || _a === void 0 ? void 0 : _a.error)
                return { error: (_b = initResult === null || initResult === void 0 ? void 0 : initResult.data) === null || _b === void 0 ? void 0 : _b.error };
            return {};
        });
    }
    callHandler(options) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const { container } = options;
            try {
                const callRunner = `http://${container.host}:${container.port}/call`;
                debug('callRunner', { callRunner, params: options });
                const result = yield axios_1.default.post(callRunner, { params: options });
                if ((_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.error)
                    return { error: (_b = result === null || result === void 0 ? void 0 : result.data) === null || _b === void 0 ? void 0 : _b.error };
                if ((_c = result === null || result === void 0 ? void 0 : result.data) === null || _c === void 0 ? void 0 : _c.resolved) {
                    return result.data.resolved;
                }
                return Promise.reject((_d = result === null || result === void 0 ? void 0 : result.data) === null || _d === void 0 ? void 0 : _d.rejected);
            }
            catch (e) {
                debug('error', e);
                return ({ error: e });
            }
        });
    }
}
exports.ContainerController = ContainerController;
//# sourceMappingURL=container-controller.js.map