"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Packager = exports.sort = void 0;
const debug_1 = __importDefault(require("debug"));
const minilinks_1 = require("./minilinks");
const debug = (0, debug_1.default)('deeplinks:packager');
const corePackage = '@deep-foundation/core';
function sort(pckg, data, errors = [], references) {
    let sorted = [];
    if (pckg.strict) {
        sorted = data;
    }
    else {
        for (let i = 0; i < data.length; i++) {
            const item = data[i];
            if (item.value && !item[references.type]) {
                const max = sorted.reduce((p, l, index) => l[references.id] == item[references.id] ? index : p, 0);
                sorted.splice((max) + 1, 0, item);
            }
            else if (item.package) {
                sorted.splice(0, 0, item);
            }
            else {
                const _ = item === null || item === void 0 ? void 0 : item._;
                const firstDependIndex = sorted.findIndex((l, index) => {
                    return l[references.from] == item[references.id] || l[references.to] == item[references.id] || ((l[references.type] == item[references.id]) && !l._);
                });
                const maxDependIndex = sorted.reduce((p, l, index) => {
                    return ((l[references.id] == item[references.type]) && !_) || l[references.id] == item[references.from] || l[references.id] === item[references.to] ? index : p;
                }, 0);
                const _index = (!!~firstDependIndex && firstDependIndex < maxDependIndex ? firstDependIndex : maxDependIndex);
                sorted.splice(_index + 1, 0, item);
            }
        }
    }
    return { sorted };
}
exports.sort = sort;
class Packager {
    constructor(client) {
        this.client = client;
        global.packager = this;
    }
    fetchPackageNamespaceId(name, deep) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const q = yield this.client.select({
                    value: { _eq: name },
                    link: { type_id: { _eq: yield deep.id('@deep-foundation/core', 'PackageNamespace') } },
                }, {
                    table: `strings`,
                    returning: 'id: link_id'
                });
                return { error: false, namespaceId: (_b = (_a = q === null || q === void 0 ? void 0 : q.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id };
            }
            catch (error) {
                debug('fetchPackageNamespaceId error');
                return { error, namespaceId: 0 };
            }
            return { error: true, namespaceId: 0 };
        });
    }
    fetchDependenciedLinkId(pckg, dependedLink) {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const packageName = (_c = (_a = pckg === null || pckg === void 0 ? void 0 : pckg.dependencies) === null || _a === void 0 ? void 0 : _a[(_b = dependedLink === null || dependedLink === void 0 ? void 0 : dependedLink.package) === null || _b === void 0 ? void 0 : _b.dependencyId]) === null || _c === void 0 ? void 0 : _c.name;
                const packageTableName = `strings`;
                const q = yield this.client.select({
                    value: (_d = dependedLink === null || dependedLink === void 0 ? void 0 : dependedLink.package) === null || _d === void 0 ? void 0 : _d.containValue,
                    link: { from: { [packageTableName]: { value: packageName } } },
                }, {
                    table: `strings`,
                    returning: 'id link { id: to_id }'
                });
                return ((_g = (_f = (_e = q === null || q === void 0 ? void 0 : q.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.link) === null || _g === void 0 ? void 0 : _g.id) || 0;
            }
            catch (error) {
                debug('fetchDependenciedLinkId error');
                console.log(error);
            }
            return 0;
        });
    }
    insertItem(items, item, errors, mutated) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            debug('insertItem', item);
            try {
                if (item.type) {
                    const insert = { id: +item.id, type_id: +item.type, from_id: +item.from || 0, to_id: +item.to || 0 };
                    const linkInsert = yield this.client.insert(insert, { name: 'IMPORT_PACKAGE_LINK' });
                    if (linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors) {
                        errors.push(linkInsert === null || linkInsert === void 0 ? void 0 : linkInsert.errors);
                    }
                }
                debug('insertItem promise', item.id);
                yield this.client.await(+item.id);
                debug('insertItem promise awaited', item.id);
                if (item.value && !item.package) {
                    debug('insertItem value', item);
                    let valueLink;
                    if (!item.type) {
                        valueLink = items.find(i => i.id === item.id && !!i.type);
                        debug('insertItem .value', item, valueLink);
                    }
                    else {
                        valueLink = item;
                    }
                    if (!valueLink) {
                        errors.push(`Link ${JSON.stringify(item)} for value not founded.`);
                    }
                    else {
                        debug('insertItem tables');
                        const type = typeof ((_a = item === null || item === void 0 ? void 0 : item.value) === null || _a === void 0 ? void 0 : _a.value);
                        const valueInsert = yield this.client.insert(Object.assign({ link_id: valueLink.id }, item.value), { table: `${type}s`, name: 'IMPORT_PACKAGE_VALUE' });
                        debug('insertItem valueInsert', valueInsert);
                        if (valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors)
                            errors.push(valueInsert === null || valueInsert === void 0 ? void 0 : valueInsert.errors);
                    }
                }
            }
            catch (error) {
                debug('insertItem error');
                console.log(error);
                errors.push(error);
            }
            return;
        });
    }
    insertItems(pckg, data, counter, dependedLinks, errors = [], mutated = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                for (let i = 0; i < dependedLinks.length; i++) {
                    const item = dependedLinks[i];
                    yield this.insertItem(data, item, errors, mutated);
                    if (errors.length)
                        return { ids: [] };
                }
                for (let i = 0; i < data.length; i++) {
                    const item = data[i];
                    if (!item.package) {
                        yield this.insertItem(data, item, errors, mutated);
                        if (errors.length)
                            return { ids: [] };
                    }
                }
                return;
            }
            catch (error) {
                debug('insertItems error');
                console.log(error);
                errors.push(error);
            }
            return;
        });
    }
    globalizeIds(pckg, ids, links) {
        return __awaiter(this, void 0, void 0, function* () {
            const difference = {};
            const global = links.map(l => (Object.assign({}, l)));
            let idsIndex = 0;
            for (let l = 0; l < links.length; l++) {
                const item = links[l];
                const oldId = item.id;
                let newId;
                if (item.package) {
                    newId = yield this.client.id(pckg.dependencies[item.package.dependencyId].name, item.package.containValue, true);
                    if (!newId)
                        pckg.errors.push(`dependency [${pckg.dependencies[item.package.dependencyId].name, item.package.containValue}], not founded`);
                }
                else if (item.type) {
                    newId = ids[idsIndex++];
                }
                if (oldId && newId)
                    difference[oldId] = newId;
                if (item.type || item.package) {
                    for (let l = 0; l < links.length; l++) {
                        const link = links[l];
                        if (link.from === oldId) {
                            global[l].from = newId;
                        }
                        if (link.to === oldId) {
                            global[l].to = newId;
                        }
                        if (link.type === oldId && !link._) {
                            global[l].type = newId;
                        }
                        if (link.id === oldId) {
                            global[l].id = newId;
                        }
                    }
                }
            }
            const resultGlobal = global.filter(l => !l.package);
            return { global: resultGlobal, difference };
        });
    }
    import(pckg) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const errors = [];
            try {
                if (!((_a = pckg === null || pckg === void 0 ? void 0 : pckg.package) === null || _a === void 0 ? void 0 : _a.name))
                    throw new Error(`!pckg?.package?.name`);
                if (!((_b = pckg === null || pckg === void 0 ? void 0 : pckg.package) === null || _b === void 0 ? void 0 : _b.version))
                    throw new Error(`!pckg?.package?.version`);
                const { data, counter, dependedLinks, packageId, namespaceId } = yield this.deserialize(pckg, errors);
                if (errors.length)
                    return { errors };
                const { sorted } = sort(pckg, data, errors, {
                    id: 'id',
                    from: 'from',
                    to: 'to',
                    type: 'type',
                });
                if (errors.length)
                    return { errors };
                const mutated = {};
                const ids = yield this.client.reserve(counter);
                const { global, difference } = yield this.globalizeIds(pckg, ids, sorted);
                if ((_c = pckg.errors) === null || _c === void 0 ? void 0 : _c.length) {
                    return { errors };
                }
                yield this.insertItems(pckg, global, counter, dependedLinks, errors, mutated);
                if (errors.length)
                    return { errors };
                return { ids, errors, namespaceId: difference[namespaceId], packageId: difference[packageId] };
            }
            catch (error) {
                debug('import error');
                errors.push(error);
            }
            return { ids: [], errors };
        });
    }
    selectLinks(options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const Contain = yield this.client.id('@deep-foundation/core', 'Contain');
            const Package = yield this.client.id('@deep-foundation/core', 'Package');
            const PackageVersion = yield this.client.id('@deep-foundation/core', 'PackageVersion');
            const result = yield this.client.select({
                _or: [
                    { id: { _eq: options.packageLinkId } },
                    { type_id: { _eq: Contain }, from: { id: { _eq: options.packageLinkId } } },
                    { in: { type_id: { _eq: Contain }, from: { id: { _eq: options.packageLinkId } } } },
                    { type_id: { _eq: PackageVersion }, to: { id: { _eq: options.packageLinkId } } },
                ]
            }, {
                name: 'LOAD_PACKAGE_LINKS',
                returning: `
          id type_id from_id to_id value
          contains: in(where: { type_id: { _eq: ${Contain} }, from: { type_id: { _eq: ${Package} } } }) {
            id value
            package: from {
              id value
            }
          }
          _type:type {
            id value
            contains: in(where: { type_id: { _eq: ${Contain} }, from: { type_id: { _eq: ${Package} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
          _from:from {
            id
            contains: in(where: { type_id: { _eq: ${Contain} }, from: { type_id: { _eq: ${Package} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
          _to:to {
            id
            contains: in(where: { type_id: { _eq: ${Contain} }, from: { type_id: { _eq: ${Package} } } }) {
              id value
              package: from {
                id value
              }
            }
          }
        `
            });
            return (0, minilinks_1.minilinks)((_a = (result)) === null || _a === void 0 ? void 0 : _a.data);
        });
    }
    deserialize(pckg, errors = []) {
        return __awaiter(this, void 0, void 0, function* () {
            const Contain = yield this.client.id('@deep-foundation/core', 'Contain');
            const Package = yield this.client.id('@deep-foundation/core', 'Package');
            const PackageNamespace = yield this.client.id('@deep-foundation/core', 'PackageNamespace');
            const Active = yield this.client.id('@deep-foundation/core', 'PackageActive');
            const Version = yield this.client.id('@deep-foundation/core', 'PackageVersion');
            const containsHash = {};
            let counter = 0;
            const dependedLinks = [];
            let packageId;
            const data = [];
            if (pckg.package.name !== corePackage) {
                packageId = 'package';
                containsHash[packageId] = ++counter;
                data.push({
                    id: packageId,
                    type: Package,
                    value: { value: pckg.package.name },
                    _: true,
                });
            }
            data.push(...pckg.data.map(l => (Object.assign(Object.assign({}, l), { value: l.value ? Object.assign({}, l.value) : undefined }))));
            for (let l = 0; l < data.length; l++) {
                const item = data[l];
                const local = item.id;
                if (containsHash[local])
                    item.id = containsHash[local];
                else
                    containsHash[local] = (item.id = ++counter);
                if (item.package)
                    dependedLinks.push(item);
            }
            for (let l = 0; l < data.length; l++) {
                const item = data[l];
                if (item.type && !item._) {
                    if (!containsHash[item.type])
                        errors.push(`${item.id} type ${item.type} !contain ${containsHash[item.type]}`);
                    item.type = containsHash[item.type];
                }
                if (item.from) {
                    if (!containsHash[item.from])
                        errors.push(`${item.id} from ${item.from} !contain ${containsHash[item.from]}`);
                    item.from = containsHash[item.from];
                }
                if (item.to) {
                    if (!containsHash[item.to])
                        errors.push(`${item.id} to ${item.to} !contain ${containsHash[item.to]}`);
                    item.to = containsHash[item.to];
                }
            }
            if (pckg.package.name === corePackage) {
                packageId = 'package';
                containsHash[packageId] = ++counter;
                data.push({
                    id: packageId,
                    type: Package,
                    value: { value: pckg.package.name },
                    _: true,
                });
            }
            const containsArray = Object.keys(containsHash);
            for (let c = 0; c < containsArray.length; c++) {
                const containKey = containsArray[c];
                if (containsHash[containKey] != containsHash[packageId] && !!data.find(l => (l === null || l === void 0 ? void 0 : l.id) === containsHash[containKey] && !(l === null || l === void 0 ? void 0 : l.package))) {
                    data.push({
                        id: ++counter,
                        type: Contain,
                        from: containsHash[packageId],
                        to: containsHash[containKey],
                        value: { value: containKey },
                        _: true,
                    });
                }
            }
            const n = yield this.fetchPackageNamespaceId(pckg.package.name, this.client);
            if (n.error)
                errors.push(n.error);
            let namespaceId = n.namespaceId;
            if (!namespaceId) {
                namespaceId = ++counter;
                data.push({
                    id: namespaceId,
                    type: PackageNamespace,
                    value: { value: pckg.package.name },
                    _: true,
                });
                if (this.client.linkId)
                    data.push({
                        id: ++counter,
                        type: Contain,
                        from: this.client.linkId,
                        to: namespaceId,
                        _: true,
                    });
                data.push({
                    id: ++counter,
                    type: Active,
                    to: containsHash[packageId],
                    from: namespaceId,
                    _: true,
                });
            }
            data.push({
                id: ++counter,
                type: Version,
                to: containsHash[packageId],
                from: namespaceId,
                value: { value: pckg.package.version },
                _: true,
            });
            data.push({
                id: ++counter,
                type: Contain,
                from: namespaceId,
                to: containsHash[packageId],
                _: true,
            });
            if (this.client.linkId)
                data.push({
                    id: ++counter,
                    type: Contain,
                    from: this.client.linkId,
                    to: containsHash[packageId],
                    _: true,
                });
            return { data, errors, counter, dependedLinks, packageId: containsHash[packageId], namespaceId };
        });
    }
    serialize(globalLinks, options, pckg) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
        return __awaiter(this, void 0, void 0, function* () {
            const Contain = yield this.client.id('@deep-foundation/core', 'Contain');
            const names = {};
            for (let l = 0; l < globalLinks.links.length; l++) {
                const globalLink = globalLinks.links[l];
                if (((_a = globalLink === null || globalLink === void 0 ? void 0 : globalLink.contains) === null || _a === void 0 ? void 0 : _a.length) > 1)
                    pckg.errors.push(`link ${globalLink.id} have more then 1 Contain from package ${JSON.stringify(globalLink === null || globalLink === void 0 ? void 0 : globalLink.contains)}`);
                if (!((globalLink === null || globalLink === void 0 ? void 0 : globalLink.type_id) === Contain && ((_b = globalLink === null || globalLink === void 0 ? void 0 : globalLink.from) === null || _b === void 0 ? void 0 : _b.id) === options.packageLinkId)
                    && (globalLink === null || globalLink === void 0 ? void 0 : globalLink.id) !== options.packageLinkId) {
                    const name = (_e = (_d = (_c = globalLink === null || globalLink === void 0 ? void 0 : globalLink.contains) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.value) === null || _e === void 0 ? void 0 : _e.value;
                    if (!name)
                        pckg.errors.push(`link ${globalLink.id} have invalid name: ${name}`);
                    if (names[name])
                        pckg.errors.push(`l ink ${globalLink.id} have duplicated in package name ${name}, duplicate link ${names[name]}`);
                    names[name] = globalLink.id;
                }
            }
            const localLinks = {};
            const lbyg = {};
            const gbyl = {};
            let localCounter = 1;
            const addLocalLink = (gId, localLink) => {
                if (!localLink.id)
                    localLink.id = localCounter++;
                gbyl[localLink.id] = gId;
                lbyg[gId] = localLink.id;
                localLinks[localLink.id] = localLink;
                return localLink.id;
            };
            for (let g = 0; g < globalLinks.links.length; g++) {
                const link = globalLinks.links[g];
                if (!((link === null || link === void 0 ? void 0 : link.type_id) === Contain && ((_f = link === null || link === void 0 ? void 0 : link.from) === null || _f === void 0 ? void 0 : _f.id) === options.packageLinkId)
                    && (link === null || link === void 0 ? void 0 : link.id) !== options.packageLinkId) {
                    const name = (_j = (_h = (_g = link === null || link === void 0 ? void 0 : link.contains) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.value) === null || _j === void 0 ? void 0 : _j.value;
                    addLocalLink(link.id, { id: name });
                }
            }
            const dependencies = [];
            const getDependencyIndex = (link) => {
                var _a, _b, _c, _d;
                const alreadyIndex = dependencies.findIndex(l => { var _a, _b, _c, _d; return l.name === ((_d = (_c = (_b = (_a = link === null || link === void 0 ? void 0 : link.contains) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.package) === null || _c === void 0 ? void 0 : _c.value) === null || _d === void 0 ? void 0 : _d.value); });
                if (!!~alreadyIndex) {
                    return alreadyIndex;
                }
                else {
                    dependencies.push({ name: (_d = (_c = (_b = (_a = link === null || link === void 0 ? void 0 : link.contains) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.package) === null || _c === void 0 ? void 0 : _c.value) === null || _d === void 0 ? void 0 : _d.value });
                    return dependencies.length - 1;
                }
            };
            const getDependencedId = (link) => {
                var _a, _b, _c;
                const dependencyId = getDependencyIndex(link);
                const containValue = (_c = (_b = (_a = link === null || link === void 0 ? void 0 : link.contains) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.value) === null || _c === void 0 ? void 0 : _c.value;
                const exists = Object.values(localLinks).find((l) => { var _a, _b; return ((_a = l === null || l === void 0 ? void 0 : l.package) === null || _a === void 0 ? void 0 : _a.dependencyId) === dependencyId && ((_b = l === null || l === void 0 ? void 0 : l.package) === null || _b === void 0 ? void 0 : _b.containValue) === containValue; });
                return exists || addLocalLink(link.id, {
                    package: { dependencyId, containValue }
                });
            };
            for (let g = 0; g < globalLinks.links.length; g++) {
                const globalLink = globalLinks.links[g];
                if (!((globalLink === null || globalLink === void 0 ? void 0 : globalLink.type_id) === Contain && ((_k = globalLink === null || globalLink === void 0 ? void 0 : globalLink.from) === null || _k === void 0 ? void 0 : _k.id) === options.packageLinkId)
                    && (globalLink === null || globalLink === void 0 ? void 0 : globalLink.id) !== options.packageLinkId) {
                    const localLink = localLinks[lbyg[globalLink.id]];
                    if (localLink) {
                        if (globalLink.type_id) {
                            if (globalLink.type) {
                                localLink.type_id = localLinks[lbyg[globalLink.type_id]];
                            }
                            else {
                                localLink.type_id = getDependencedId(globalLink._type);
                            }
                        }
                        if (globalLink.from_id) {
                            if (globalLink.from) {
                                localLink.from_id = localLinks[lbyg[globalLink.from_id]];
                            }
                            else {
                                localLink.from_id = getDependencedId(globalLink._from);
                            }
                        }
                        if (globalLink.to_id) {
                            if (globalLink.to) {
                                localLink.to_id = localLinks[lbyg[globalLink.to_id]];
                            }
                            else {
                                localLink.to_id = getDependencedId(globalLink._to);
                            }
                        }
                        if (globalLink.value) {
                            localLink.value = (_l = globalLink === null || globalLink === void 0 ? void 0 : globalLink.value) === null || _l === void 0 ? void 0 : _l.value;
                        }
                    }
                }
            }
            pckg.data = Object.values(localLinks).map((l) => (l.type_id ? {
                id: l.id,
                type: l.type_id,
                from: l.from_id,
                to: l.to_id,
                value: l.value,
            } : Object.assign({}, l)));
            pckg.dependencies = dependencies;
            return pckg;
        });
    }
    export(options) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            const globalLinks = yield this.selectLinks(options);
            const Package = yield this.client.id('@deep-foundation/core', 'Package');
            const PackageVersion = yield this.client.id('@deep-foundation/core', 'PackageVersion');
            const version = ((_a = globalLinks.types[PackageVersion]) === null || _a === void 0 ? void 0 : _a[0]) || '0.0.0';
            const pack = (_b = globalLinks.types[Package]) === null || _b === void 0 ? void 0 : _b[0];
            const pckg = {
                package: { name: (_c = pack === null || pack === void 0 ? void 0 : pack.value) === null || _c === void 0 ? void 0 : _c.value, version: (_d = version === null || version === void 0 ? void 0 : version.value) === null || _d === void 0 ? void 0 : _d.value },
                data: [],
                errors: [],
            };
            if (!pack)
                pckg.errors.push('!package');
            if (!(pckg === null || pckg === void 0 ? void 0 : pckg.package.name))
                pckg.errors.push('!pckg?.package.name');
            if (!(pckg === null || pckg === void 0 ? void 0 : pckg.package.version))
                pckg.errors.push('!pckg?.package.version');
            if (!version)
                pckg.errors.push('!version');
            if ((_e = pckg === null || pckg === void 0 ? void 0 : pckg.errors) === null || _e === void 0 ? void 0 : _e.length)
                return pckg;
            const ml = (0, minilinks_1.minilinks)(globalLinks.links.filter(l => l.id !== (version === null || version === void 0 ? void 0 : version.id)));
            const { sorted } = sort(pckg, ml.links, pckg.errors, {
                id: 'id',
                from: 'from_id',
                to: 'to_id',
                type: 'type_id',
            });
            ml.links = sorted;
            yield this.serialize(ml, options, pckg);
            return pckg;
        });
    }
}
exports.Packager = Packager;
//# sourceMappingURL=packager.js.map