"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const debug_1 = __importDefault(require("debug"));
const client_1 = require("@deep-foundation/hasura/client");
const api_1 = require("@deep-foundation/hasura/api");
const client_2 = require("../client");
const links_1 = require("./links");
const bool_exp_to_sql_1 = require("../bool_exp_to_sql");
const SCHEMA = 'public';
const debug = (0, debug_1.default)('deeplinks:eh:values');
exports.api = new api_1.HasuraApi({
    path: process.env.DEEPLINKS_HASURA_PATH,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const client = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({
    apolloClient: client,
});
const portHash = {};
const containerHash = {};
exports.default = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g;
    try {
        const event = (_a = req === null || req === void 0 ? void 0 : req.body) === null || _a === void 0 ? void 0 : _a.event;
        const operation = event === null || event === void 0 ? void 0 : event.op;
        if (operation === 'INSERT' || operation === 'UPDATE' || operation === 'DELETE') {
            let oldValueRow = (_b = event === null || event === void 0 ? void 0 : event.data) === null || _b === void 0 ? void 0 : _b.old;
            let newValueRow = (_c = event === null || event === void 0 ? void 0 : event.data) === null || _c === void 0 ? void 0 : _c.new;
            const linkId = (_d = newValueRow === null || newValueRow === void 0 ? void 0 : newValueRow.link_id) !== null && _d !== void 0 ? _d : oldValueRow === null || oldValueRow === void 0 ? void 0 : oldValueRow.link_id;
            const linkRow = (_f = (_e = (yield deep.select({
                id: { _eq: linkId },
            }, {
                returning: `id from_id type_id to_id`,
            }))) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f[0];
            const oldRow = Object.assign(Object.assign({}, linkRow), { value: oldValueRow });
            const newRow = Object.assign(Object.assign({}, linkRow), { value: newValueRow });
            debug('operation', operation);
            debug('oldValueRow', oldValueRow);
            debug('newValueRow', newValueRow);
            debug('oldRow', oldRow);
            if (oldValueRow && !newValueRow) {
                yield deep.delete({
                    link_id: { _eq: oldValueRow.link_id },
                }, { table: 'bool_exp' });
            }
            if (newValueRow && newRow.type_id === (yield deep.id('@deep-foundation/core', 'BoolExp'))) {
                yield (0, bool_exp_to_sql_1.boolExpToSQL)(newRow.id, (_g = newRow === null || newRow === void 0 ? void 0 : newRow.value) === null || _g === void 0 ? void 0 : _g.value);
            }
            try {
                yield (0, links_1.handleOperation)('Update', oldRow, newRow);
                return res.status(200).json({});
            }
            catch (error) {
                throw error;
            }
            return res.status(500).json({ error: 'notexplained' });
        }
        return res.status(500).json({ error: 'operation can be only INSERT or UPDATE' });
    }
    catch (error) {
        return res.status(500).json({ error: error.toString() });
    }
});
//# sourceMappingURL=values.js.map